/* drivers/video/tomtom/a043fw03.c
 *
 * Specific functionality for the AUO A043FW03 screen. 
 *
 * ######## NOTE #############
 * Based on preliminary datasheet A043FW03 V1 ver 0.1 (15-05-2009)
 *
 * ###########################
 *
 * Copyright (C) 2009 TomTom BV <http://www.tomtom.com/>
 * Author: Will Lin <Will.Lin@tomtom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifdef __KERNEL__
#include <linux/delay.h>
#include <linux/cpufreq.h>
#include <barcelona/gopins.h>
#include <linux/err.h>
#include <asm/hardware/clock.h>
#endif /* __KERNEL */
#ifdef __BOOTLOADER__
#include "timer.h"
#include "gopins.h"
#endif /* __BOOTLOADER__ */
#include "lcdregs.h"
#include "screeninfo.h"
#include "framerate.h"
#include "a043fw03.h"

#define USE_DATA_ENABLE

#define FRAME   17 //unit ms. By typical frame rate 60Hz

// LCDCON1
#define MMODE		0
#define PNR_MODE	3
#define BPP_MODE	12
#define ENVID		0

// LCDCON2
#define VBPD		1  //Typical Tvbp = 12 = (VSPW + 1) + (VBPD+1)
#define VFPD		3  //Typical Tvfp = 4 = VFPD + 1
#define VSPW		9  //Typical Tvw = 10 = VSPW + 1

// LCDCON3
#define HBPD		41 //Typical Thbp = 43 = (HSPW+1) + (HBPD+1)
#define HFPD		7  //Typical Thfp = 8 = HFPD+1

// LCDCON4
#define MVAL		0
#define HSPW		0  //Typical Thw = 1 = HSPW+1

// LCDCON5
#define BPP24BL		0
#define FRM565		1
#define INVVCLK		0 //The video data is fetched at VCLK falling edge
#define INVVLINE	1
#define INVVFRAME	1
#define INVVD		0
#define INVVDEN		0
#define INVPWREN	0
#define INVLEND		1
#define PWREN		1
#define ENLEND		0
#define BSWP		0
#define HWSWP		1

void A043FW03_Init(void)
{
	unsigned long int	divider=(unsigned long int) get_clkval( &ttgfb_a043fw03_lcdspecs,
									ttgfb_a043fw03_lcdspecs.refresh_max,
									ttgfb_a043fw03_lcdspecs.curr_hclk );
	unsigned long int	hozval;
	unsigned long int	lineval;

	/* Get HOZVAL and LINEVAL. */
	get_hozlineval( &ttgfb_a043fw03_lcdspecs, &hozval, &lineval );

	/* Start display controller */
	rLCDCON1 &= ~1;
	rTCONSEL = 0;
	rLCDCON1 = (divider << 8) | (MMODE << 7) | (((unsigned long int) ttgfb_a043fw03_lcdspecs.pnr_mode) << 5) |
						   (((unsigned long int) ttgfb_a043fw03_lcdspecs.bpp_mode) << 1) | (ENVID << 0);
	rLCDCON2 = (((unsigned long int) ttgfb_a043fw03_lcdspecs.screen.tft.vbpd) << 24) |
		   (lineval << 14) |
		   (((unsigned long int) ttgfb_a043fw03_lcdspecs.screen.tft.vfpd) << 6) |
		   (((unsigned long int) ttgfb_a043fw03_lcdspecs.screen.tft.vspw) << 0);
	rLCDCON3 = (((unsigned long int) ttgfb_a043fw03_lcdspecs.screen.tft.hbpd) << 19) |
		   (hozval << 8) |
		   (((unsigned long int) ttgfb_a043fw03_lcdspecs.screen.tft.hfpd) << 0);
	rLCDCON4 = (MVAL << 8) | (((unsigned long int) ttgfb_a043fw03_lcdspecs.screen.tft.hspw) << 0);
	rLCDCON5 = (BPP24BL << 12) | (FRM565 << 11) | (INVVCLK << 10) | (INVVLINE << 9) | (INVVFRAME << 8) | (INVVD << 7) |
		   (INVVDEN << 6) | (INVPWREN << 5) | (INVLEND << 4) | (PWREN << 3) | (ENLEND << 2) | (BSWP << 1) | (HWSWP << 0);

	// Start display controller
	rLCDCON1 |= 1;

#ifndef USE_DATA_ENABLE  
    IO_Deactivate(VDEN);    
#endif
           	
	mdelay(1*FRAME);
	IO_Deactivate(LCD_CS); //Pull High DISP
	mdelay(9*FRAME);
}

void A043FW03_Off(void)
{
	IO_Activate(LCD_CS); //Pull Low DISP	
	mdelay(7*FRAME);
	// Stop display controller
	rLCDCON1 = 0;

}

#if defined CONFIG_CPU_FREQ && defined CONFIG_S3C24XX_DFS_CPUFREQ 
struct lcd_screen_info ttgfb_a043fw03_lcdspecs = {"AUO", "A043FW03", 95, 54, 480, 272, 60, 70, A043FW03_Init, A043FW03_Off, NULL,
						  PNR_TFT, BPP_16BPP_TFT, 0, {{VSPW, VBPD, VFPD, HSPW, HBPD, HFPD}},
						  lcd_def_freq_policy, lcd_def_freq_trans};
#else
struct lcd_screen_info ttgfb_a043fw03_lcdspecs = {"AUO", "A043FW03", 95, 54, 480, 272, 60, 70, A043FW03_Init, A043FW03_Off, NULL,
						  PNR_TFT, BPP_16BPP_TFT, 0, {{VSPW, VBPD, VFPD, HSPW, HBPD, HFPD}}};
#endif
/* EOF */
